package org.artifactory.repo.system.suppliers;

import lombok.extern.slf4j.Slf4j;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.descriptor.config.MutableCentralConfigDescriptor;
import org.artifactory.descriptor.property.PropertySet;
import org.artifactory.descriptor.repo.LocalRepoDescriptor;

import java.util.ArrayList;
import java.util.List;

import static org.artifactory.descriptor.property.PropertySet.ARTIFACTORY_RESERVED_PROP_SET;
import static org.artifactory.repo.config.RepoConfigDefaultValues.DEFAULT_BLACKED_OUT;
import static org.artifactory.repo.config.RepoConfigDefaultValues.DEFAULT_INCLUDES_PATTERN;

/**
 * @author Tomer Slutsky
 */
@Slf4j
public class EdgeRepo extends SystemRepoBase {
    public EdgeRepo(CentralConfigService centralConfigService) {
        super(centralConfigService);
    }

    @Override
    public LocalRepoDescriptor buildDescriptor(String repoKey) {
        LocalRepoDescriptor descriptor = super.buildDescriptor(repoKey);
        descriptor.setDescription("A generic repository for uploads");
        descriptor.setIncludesPattern(DEFAULT_INCLUDES_PATTERN);
        descriptor.setBlackedOut(DEFAULT_BLACKED_OUT);

        return descriptor;
    }

    @Override
    public void addToRepoList(MutableCentralConfigDescriptor mutableDescriptor, LocalRepoDescriptor descriptor) {
        descriptor.setPropertySets(getPropertySets(mutableDescriptor));
        super.addToRepoList(mutableDescriptor, descriptor);
        mutableDescriptor.conditionallyAddToBackups(descriptor);
    }

    private List<PropertySet> getPropertySets(MutableCentralConfigDescriptor mutableDescriptor) {
        PropertySet defaultPropertySet = mutableDescriptor.getPropertySets().stream()
                .filter(propertySet -> propertySet.getName().equals(ARTIFACTORY_RESERVED_PROP_SET))
                .findFirst()
                .orElse(null);
        if (defaultPropertySet == null) {
            return null;
        }
        List<PropertySet> propertySets = new ArrayList<>();
        propertySets.add(defaultPropertySet);
        return propertySets;
    }
}
