package org.artifactory.queue.behaviors;

import org.apache.commons.compress.utils.Lists;
import org.artifactory.storage.event.EventInfo;

import java.util.List;

import static org.artifactory.storage.event.EventType.create;
import static org.artifactory.storage.event.EventType.delete;
import static org.artifactory.storage.event.EventType.props;
import static org.artifactory.storage.event.EventType.update;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertThrows;

/**
 * @author avishayh
 */
public class SimilarPathDedupBehaviourTest extends AbstractEventReductionBehaviorTest {

    public void basicDedupBehaviour() {
        SimilarPathDedupBehavior similarPathDedupBehaviour = new SimilarPathDedupBehavior();
        List<EventInfo> before = Lists.newArrayList();
        add(before, create, "a");
        add(before, props, "a");
        add(before, props, "a");
        add(before, create, "b");
        add(before, props, "a");
        add(before, props, "a");
        add(before, create, "c");
        add(before, props, "a");
        List<EventInfo> after = similarPathDedupBehaviour.apply(before);
        assertEquals(after.size(), 4);
        assertEquals(after.get(0).toString(), "1|create|a");
        assertEquals(after.get(1).toString(), "4|create|b");
        assertEquals(after.get(2).toString(), "7|create|c");
        assertEquals(after.get(3).toString(), "8|props|a");
    }

    public void treat2EventTypesAsSame() {
        SimilarPathDedupBehavior similarPathDedupBehaviour = new SimilarPathDedupBehavior(create, props);
        List<EventInfo> before = Lists.newArrayList();
        add(before, create, "a");
        add(before, props, "a");
        add(before, props, "a");
        add(before, create, "b");
        add(before, props, "a");
        add(before, props, "a");
        add(before, create, "c");
        add(before, update, "c");
        add(before, props, "a");
        List<EventInfo> after = similarPathDedupBehaviour.apply(before);
        assertEquals(after.size(), 4);
        assertEquals(after.get(0).toString(), "4|create|b");
        assertEquals(after.get(1).toString(), "7|create|c");
        assertEquals(after.get(2).toString(), "8|update|c");
        assertEquals(after.get(3).toString(), "9|props|a");
    }

    public void treat3EventTypesAsSame() {
        SimilarPathDedupBehavior similarPathDedupBehaviour = new SimilarPathDedupBehavior(create, update, props);
        List<EventInfo> before = Lists.newArrayList();
        add(before, create, "a");
        add(before, props, "a");
        add(before, props, "a");
        add(before, create, "b");
        add(before, update, "b");
        add(before, props, "a");
        add(before, props, "a");
        add(before, update, "a");
        add(before, update, "c");
        add(before, props, "c");
        add(before, props, "a");
        List<EventInfo> after = similarPathDedupBehaviour.apply(before);
        assertEquals(after.size(), 3);
        assertEquals(after.get(0).toString(), "5|update|b");
        assertEquals(after.get(1).toString(), "10|props|c");
        assertEquals(after.get(2).toString(), "11|props|a");
    }

    public void illegalDedupBehaviour() {
        //either 0 or more than 1 types allowed
        assertThrows(IllegalArgumentException.class, () -> new SimilarPathDedupBehavior(create));
        assertThrows(IllegalArgumentException.class, () -> new SimilarPathDedupBehavior(update));
        assertThrows(IllegalArgumentException.class, () -> new SimilarPathDedupBehavior(delete));
        assertThrows(IllegalArgumentException.class, () -> new SimilarPathDedupBehavior(props));
    }
}
