package org.artifactory.resource;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.lang.reflect.Parameter;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.IntStream;

/**
 * @author Omri Ziv
 */
class RemoteRepoResourceTest {

    @Test
    void testConstructorFitToReConstruct() {
        Constructor<?>[] constructors = RemoteRepoResource.class.getConstructors();
        List<Parameter> constructorParams = Arrays.asList(constructors[0].getParameters());

        Optional<Method> optionalMethod = Arrays.stream(RemoteRepoResource.class.getDeclaredMethods())
                .filter(method1 -> method1.getName().equals("reConstruct")
                        && method1.getParameterCount() > 1)
                .findFirst();
        Assertions.assertTrue(optionalMethod.isPresent());
        Method reconstructMethod = optionalMethod.get();

        List<Parameter> reConstructParams = Arrays.asList(reconstructMethod.getParameters());
        Assertions.assertEquals("repoPath", constructorParams.get(0).getName());
        Assertions.assertEquals(reConstructParams.size(), constructorParams.size() - 1,
                "Constructor fields should match Reconstruct fields");
        IntStream.range(0, reConstructParams.size())
                .forEach(index -> {
                    Assertions.assertEquals(reConstructParams.get(index).getName(),
                            constructorParams.get(index + 1).getName(),
                            String.format("Field number %d with name '%s' on " +
                                            "reConstruct doesn't match '%s' on Constructor."
                                    , index, reConstructParams.get(index).getName(),
                                    constructorParams.get(index + 1).getName()));
                    Assertions.assertEquals(reConstructParams.get(index).getType(),
                            constructorParams.get(index + 1).getType(),
                            String.format("Field number %d with type '%s' on " +
                                            "reConstruct doesn't match '%s' on Constructor."
                                    , index, reConstructParams.get(index).getType().toString(),
                                    constructorParams.get(index + 1).getType().toString()));
                });
    }
}
