package org.artifactory.storage.db.aql.sql.builder.query.aql;

import org.artifactory.aql.model.AqlDomainEnum;
import org.artifactory.aql.model.AqlValue;
import org.artifactory.aql.model.AqlVariableTypeEnum;
import org.artifactory.storage.db.aql.sql.builder.query.sql.SqlTable;
import org.artifactory.storage.db.aql.sql.model.SqlTableEnum;
import org.testng.annotations.Test;

import java.util.ArrayList;
import java.util.List;

import static org.testng.Assert.assertEquals;

public class ComplexPropertyCriterionSubSelectTest {

    @Test
    public void testNotMatchFromBuildModulesToBuildProps() {
        ComplexPropertyCriterionSubSelect newCriteria =
                new ComplexPropertyCriterionSubSelect(
                        List.of(AqlDomainEnum.modules, AqlDomainEnum.builds,
                                AqlDomainEnum.buildProperties),
                        new AqlValue(AqlVariableTypeEnum.string, "start"),
                        new SqlTable(SqlTableEnum.build_props, 100),
                        "$nmatch", new AqlValue(AqlVariableTypeEnum.string, "1"),
                        new SqlTable(SqlTableEnum.build_modules, -1), false);
        String expectedValue =
                "( b.build_id not in ( select bpsub.build_id from build_props bpsub where bpsub.build_id = b" +
                        ".build_id and bpsub.prop_key = ? and bpsub.prop_value Like ? )) ";
        assertEquals(newCriteria.toSql(new ArrayList()),expectedValue);
    }

    @Test
    public void testNotMatchFromBuildPropsToBuildProps() {
        ComplexPropertyCriterionSubSelect newCriteria =
                new ComplexPropertyCriterionSubSelect(
                        List.of(AqlDomainEnum.buildProperties),
                        new AqlValue(AqlVariableTypeEnum.string, "start"), new SqlTable(SqlTableEnum.build_props, 100),
                        "$nmatch", new AqlValue(AqlVariableTypeEnum.string, "1"),
                        new SqlTable(SqlTableEnum.build_props, 100), false);
        String expectedValue =
                "( bp.build_id not in ( select bpsub.build_id from build_props bpsub where bpsub.build_id = bp" +
                        ".build_id and bpsub.prop_key = ? and bpsub.prop_value Like ? )) ";
        assertEquals(expectedValue, newCriteria.toSql(new ArrayList()));
    }

    @Test
    public void testNotEqualFromBuildModulesToBuildProps() {

        ComplexPropertyCriterionSubSelect newCriteria =
                new ComplexPropertyCriterionSubSelect(
                        List.of(AqlDomainEnum.modules, AqlDomainEnum.builds,
                                AqlDomainEnum.buildProperties),
                        new AqlValue(AqlVariableTypeEnum.string, "start"), new SqlTable(SqlTableEnum.build_props, 100),
                        "$ne", new AqlValue(AqlVariableTypeEnum.string, "1"),
                        new SqlTable(SqlTableEnum.build_props, 100), false);
        String expectedValue =
                "( b.build_id not in ( select bpsub.build_id from build_props bpsub where bpsub.build_id = b" +
                        ".build_id and bpsub.prop_key = ? and bpsub.prop_value = ? )) ";
        assertEquals(expectedValue, newCriteria.toSql(new ArrayList()));
    }

    @Test
    public void testNotEqualFromBuildPropertiesToBuildProps() {

        ComplexPropertyCriterionSubSelect newCriteria =
                new ComplexPropertyCriterionSubSelect(
                        List.of(AqlDomainEnum.buildProperties),
                        new AqlValue(AqlVariableTypeEnum.string, "start"), new SqlTable(SqlTableEnum.build_props, 100),
                        "$ne", new AqlValue(AqlVariableTypeEnum.string, "1"),
                        new SqlTable(SqlTableEnum.build_props, 100), false);
        String expectedValue =
                "( bp.build_id not in ( select bpsub.build_id from build_props bpsub where bpsub.build_id = bp" +
                        ".build_id and bpsub.prop_key = ? and bpsub.prop_value = ? )) ";
        assertEquals(expectedValue, newCriteria.toSql(new ArrayList()));
    }

    @Test
    public void testNotEqualFromNodePropertiesToNodeProperties() {

        ComplexPropertyCriterionSubSelect newCriteria =
                new ComplexPropertyCriterionSubSelect(
                        List.of(AqlDomainEnum.properties),
                        new AqlValue(AqlVariableTypeEnum.string, "start"), new SqlTable(SqlTableEnum.node_props, 100),
                        "$ne", new AqlValue(AqlVariableTypeEnum.string, "1"),
                        new SqlTable(SqlTableEnum.node_props, 100), false);
        String expectedValue =
                "( np.node_id not in ( select npsub.node_id from node_props npsub where npsub.node_id = np" +
                        ".node_id and npsub.prop_key = ? and npsub.prop_value = ? )) ";
        assertEquals(expectedValue, newCriteria.toSql(new ArrayList()));
    }


    @Test
    public void testNotMatchFromNodesToNodesProps() {
        ComplexPropertyCriterionSubSelect newCriteria =
                new ComplexPropertyCriterionSubSelect(
                        List.of(AqlDomainEnum.items, AqlDomainEnum.properties),
                        new AqlValue(AqlVariableTypeEnum.string, "start"), new SqlTable(SqlTableEnum.node_props, 100),
                        "$nmatch", new AqlValue(AqlVariableTypeEnum.string, "df"),
                        new SqlTable(SqlTableEnum.node_props, 100), false);
        String expectedValue =
                "( n.node_id not in ( select npsub.node_id from node_props npsub where npsub.node_id = n.node_id and " +
                        "npsub" +
                        ".prop_key = ? and npsub.prop_value Like ? )) ";

        assertEquals(expectedValue, newCriteria.toSql(new ArrayList()));
    }

    @Test
    public void testNotEqualFromNodesToNodesProps() {
        ComplexPropertyCriterionSubSelect newCriteria =
                new ComplexPropertyCriterionSubSelect(
                        List.of(AqlDomainEnum.items, AqlDomainEnum.properties),
                        new AqlValue(AqlVariableTypeEnum.string, "start"), new SqlTable(SqlTableEnum.node_props, 100),
                        "$ne", new AqlValue(AqlVariableTypeEnum.string, "df"),
                        new SqlTable(SqlTableEnum.node_props, 100), false);
        String expectedValue =
                "( n.node_id not in ( select npsub.node_id from node_props npsub where npsub.node_id = n.node_id and " +
                        "npsub" +
                        ".prop_key = ? and npsub.prop_value = ? )) ";

        assertEquals(expectedValue, newCriteria.toSql(new ArrayList()));
    }
}
