package org.artifactory.storage.db.upgrades.itest.version;

import org.artifactory.storage.db.upgrades.common.UpgradeBaseTest;
import org.artifactory.storage.db.version.ArtifactoryDBVersion;
import org.jfrog.converter.model.ChangeLogContext;
import org.jfrog.converter.predefined.PreDefinedDbConverter;
import org.testng.annotations.Test;

import java.io.IOException;
import java.sql.SQLException;

import static org.artifactory.storage.db.version.ArtifactoryDBVersion.v43;
import static org.artifactory.storage.db.version.ArtifactoryDBVersion.v44;
import static org.jfrog.converter.util.PreDefinedConverters.alterColumnSize;
import static org.jfrog.storage.util.DbUtils.withConnection;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertFalse;
import static org.testng.Assert.assertTrue;

/**
 * @author Shay Bagants
 */
@Test
public class V44ChangeTasksContextSizeTest extends UpgradeBaseTest {

    @Test
    public void testConvertColumn() throws IOException, SQLException {
        resetToVersion(v43);
        String columnName = "task_context";
        String tableName = "tasks";
        assertTrue(columnExists(tableName, columnName));
        int columnSize = withConnection(jdbcHelper, conn -> columnSize(tableName, columnName));
        assertEquals(columnSize, 1024);
        PreDefinedDbConverter v220ConversionPredicate =
                alterColumnSize("tasks", "task_context", 2048);
        boolean shouldConvert;
        try (ChangeLogContext context = new ChangeLogContext(jdbcHelper.getDataSource().getConnection(),
                dbProperties.getDbType(), v43)) {
            shouldConvert =
                    v220ConversionPredicate.testConditions(context);
        }
        assertTrue(shouldConvert);

        final ArtifactoryDBVersion dbVersion = v44;
        try (ChangeLogContext context = new ChangeLogContext(jdbcHelper.getDataSource().getConnection(),
                dbProperties.getDbType(), dbVersion)) {
            dbVersion.converter().convert(context);
            shouldConvert = v220ConversionPredicate.testConditions(context);
        }
        assertFalse(shouldConvert);
        columnSize = withConnection(jdbcHelper, conn -> columnSize(tableName, columnName));
        assertEquals(columnSize, 2048);
    }

    @Override
    protected ArtifactoryDBVersion upToVersion() {
        return ArtifactoryDBVersion.v44;
    }
}
