package org.artifactory.ui.rest.resource.admin.configuration.repositories;

import org.artifactory.api.security.AuthorizationService;
import org.artifactory.rest.common.resource.BaseResource;
import org.artifactory.ui.rest.model.admin.configuration.repository.federation.FederationConfigModel;
import org.artifactory.ui.rest.service.admin.configuration.ConfigServiceFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import javax.annotation.security.RolesAllowed;
import javax.ws.rs.Consumes;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

import static org.artifactory.ui.rest.resource.admin.configuration.repositories.RepoResourceConstants.PATH_FEDERATED;

/**
 * @author Alexei Vainshtein
 */
@Component
@Path(PATH_FEDERATED)
@Scope(BeanDefinition.SCOPE_PROTOTYPE)
@Produces(MediaType.APPLICATION_JSON)
@RolesAllowed({AuthorizationService.ROLE_ADMIN})
public class FederatedRepoUIResource extends BaseResource {

    @Autowired
    protected ConfigServiceFactory configServiceFactory;

    @POST
    @Path("validateMirror")
    @Consumes(MediaType.APPLICATION_JSON)
    @RolesAllowed({AuthorizationService.ROLE_ADMIN})
    public Response validateMirror(FederationConfigModel federationConfigModel) {
        return runService(configServiceFactory.validateFederatedRepoMirrors(), federationConfigModel);
    }

    @POST
    @Path("findMirrorRepos")
    @Consumes(MediaType.APPLICATION_JSON)
    @RolesAllowed({AuthorizationService.ROLE_ADMIN})
    public Response findMirrorRepos(FederationConfigModel federationConfigModel) {
        return runService(configServiceFactory.discoverMirrorRepos(), federationConfigModel);
    }

    @POST
    @Path("migrate/{repoKey}")
    @Consumes(MediaType.APPLICATION_JSON)
    @RolesAllowed({AuthorizationService.ROLE_ADMIN})
    public Response migrate(@PathParam("repoKey") String repoKey) {
        return runService(configServiceFactory.migrateLocalToFederatedRepository(), repoKey);
    }
}
