package org.artifactory.ui.rest.service.admin.configuration.general;

import org.artifactory.addon.AddonsManager;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.descriptor.addon.AddonSettings;
import org.artifactory.descriptor.config.CentralConfigDescriptor;
import org.artifactory.descriptor.distribution.ReleaseBundlesConfig;
import org.artifactory.descriptor.download.DownloadRedirectConfigDescriptor;
import org.artifactory.descriptor.download.FolderDownloadConfigDescriptor;
import org.artifactory.descriptor.replication.GlobalReplicationsConfigDescriptor;
import org.artifactory.descriptor.signature.SignedUrlConfig;
import org.artifactory.descriptor.sumologic.SumoLogicConfigDescriptor;
import org.artifactory.descriptor.trashcan.TrashcanConfigDescriptor;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.ArtifactoryRestResponse;
import org.artifactory.spring.ArtifactoryApplicationContext;
import org.artifactory.test.ArtifactoryHomeBoundJUnitTest;
import org.artifactory.ui.rest.model.admin.configuration.generalconfig.BasicGeneralConfig;
import org.artifactory.ui.rest.model.admin.configuration.generalconfig.GeneralConfig;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;


public class GetGeneralConfigServiceTest extends ArtifactoryHomeBoundJUnitTest {

    private GetGeneralConfigService getGeneralConfigService;
    private AutoCloseable autoCloseable;
    @Mock
    CentralConfigService centralConfigService;
    @Mock
    AuthorizationService authorizationService;
    @Mock
    ArtifactoryApplicationContext applicationContext;
    @Mock
    AddonsManager addonsManager;

    @BeforeEach
    public void setUp() {
        autoCloseable = MockitoAnnotations.openMocks(this);
        getGeneralConfigService = Mockito.spy(new GetGeneralConfigService());
        getGeneralConfigService.setCentralConfigService(centralConfigService);
        getGeneralConfigService.setAuthorizationService(authorizationService);
        createConfigMock();
        when(applicationContext.beanForType(AddonsManager.class)).thenReturn(addonsManager);
        ArtifactoryContextThreadBinder.bind(applicationContext);
    }

    @Test
    public void testProjectAdmin() {
        Mockito.when(authorizationService.isAdmin()).thenReturn(false);
        ArtifactoryRestRequest restRequest = mock(ArtifactoryRestRequest.class);
        ArtifactoryRestResponse response = new ArtifactoryRestResponse();

        getGeneralConfigService.execute(restRequest, response);
        Object result = response.getIModel();
        Assertions.assertNotNull(result);
        Assertions.assertTrue(result instanceof BasicGeneralConfig);
    }

    @Test
    public void testSuperAdmin() {
        Mockito.when(authorizationService.isAdmin()).thenReturn(true);
        ArtifactoryRestRequest restRequest = mock(ArtifactoryRestRequest.class);
        ArtifactoryRestResponse response = new ArtifactoryRestResponse();
        getGeneralConfigService.execute(restRequest, response);
        Object result = response.getIModel();
        Assertions.assertNotNull(result);
        Assertions.assertTrue(result instanceof GeneralConfig);
    }

    private void createConfigMock() {
        CentralConfigDescriptor configDescriptor = Mockito.mock(CentralConfigDescriptor.class);
        AddonSettings addonSettings = Mockito.mock(AddonSettings.class);
        Mockito.when(configDescriptor.getAddons()).thenReturn(addonSettings);
        FolderDownloadConfigDescriptor folderDownloadDescriptor = Mockito.mock(FolderDownloadConfigDescriptor.class);
        Mockito.when(configDescriptor.getFolderDownloadConfig()).thenReturn(folderDownloadDescriptor);
        TrashcanConfigDescriptor trashcanConfigDescriptor = Mockito.mock(TrashcanConfigDescriptor.class);
        Mockito.when(configDescriptor.getTrashcanConfig()).thenReturn(trashcanConfigDescriptor);
        GlobalReplicationsConfigDescriptor replicationsConfig = Mockito.mock(GlobalReplicationsConfigDescriptor.class);
        Mockito.when(configDescriptor.getReplicationsConfig()).thenReturn(replicationsConfig);
        SumoLogicConfigDescriptor sumoLogicConfig = Mockito.mock(SumoLogicConfigDescriptor.class);
        Mockito.when(configDescriptor.getSumoLogicConfig()).thenReturn(sumoLogicConfig);
        ReleaseBundlesConfig releaseBundlesConfig = Mockito.mock(ReleaseBundlesConfig.class);
        Mockito.when(configDescriptor.getReleaseBundlesConfig()).thenReturn(releaseBundlesConfig);
        SignedUrlConfig signedUrlConfig = Mockito.mock(SignedUrlConfig.class);
        Mockito.when(configDescriptor.getSignedUrlConfig()).thenReturn(signedUrlConfig);
        DownloadRedirectConfigDescriptor downloadRedirectConfig = Mockito.mock(DownloadRedirectConfigDescriptor.class);
        Mockito.when(configDescriptor.getDownloadRedirectConfig()).thenReturn(downloadRedirectConfig);
        when(configDescriptor.getSubscriptionConfig()).thenReturn(null);
        Mockito.when(centralConfigService.getDescriptor()).thenReturn(configDescriptor);
    }

    @AfterEach
    void tearDown() throws Exception {
        if (autoCloseable != null) {
            autoCloseable.close();
            autoCloseable = null;
        }
        ArtifactoryContextThreadBinder.unbind();
    }
}